@@ -6,24 +6,31 @@ licence as PoorWSGI. So enjoy it ;)
 
 from wsgiref.simple_server import make_server
 from base64 import decodebytes, encodebytes
-from json import dumps
 from collections import OrderedDict
-from io import FileIO as file
+from io import FileIO as file, BytesIO
 from sys import path as python_path
-from tempfile import TemporaryFile
 from functools import wraps
 
 import os
 import logging as log
 
-EXAMPLES_PATH = os.path.dirname(__file__)           # noqa
-python_path.insert(0, os.path.abspath(              # noqa
+EXAMPLES_PATH = os.path.dirname(__file__)
+python_path.insert(0, os.path.abspath(
     os.path.join(EXAMPLES_PATH, os.path.pardir)))
 
-from poorwsgi import Application, state, request, redirect
-from poorwsgi.session import PoorSession
-from poorwsgi.response import Response, RedirectResponse, FileResponse, \
-    JSONResponse, JSONGeneratorResponse, EmptyResponse, HTTPException
+# pylint: disable=import-error, wrong-import-position
+from poorwsgi import Application, state, request, redirect  # noqa
+from poorwsgi.session import PoorSession, SessionError  # noqa
+from poorwsgi.response import Response, RedirectResponse, \
+    FileObjResponse, FileResponse, GeneratorResponse, \
+    EmptyResponse, HTTPException # noqa
+
+try:
+    import uwsgi  # type: ignore
+
+except ModuleNotFoundError:
+    uwsgi = None  # pylint: disable=invalid-name
+
 
 logger = log.getLogger()
 logger.setLevel("DEBUG")
@@ -34,6 +41,10 @@ app.document_index = True
 app.secret_key = os.urandom(32)     # random key each run
 
 
+class MyValueError(ValueError):
+    """My value error"""
+
+
 class Storage(file):
     def __init__(self, directory, filename):
         log.debug("directory: %s; filename: %s", directory, filename)
@@ -42,7 +53,7 @@ class Storage(file):
         if os.access(self.path, os.F_OK):
             raise Exception("File %s exist yet" % filename)
 
-        super(Storage, self).__init__(self.path, 'w+b')
+        super().__init__(self.path, 'w+b')
 
 
 class StorageFactory:
@@ -52,34 +63,36 @@ class StorageFactory:
             os.mkdir(directory)
 
     def create(self, filename):
-        if filename:
-            return Storage(self.directory, filename)
-        return TemporaryFile("wb+")
+        """Create file in directory."""
+        if not filename:
+            return BytesIO()
+        return Storage(self.directory, filename)
 
 
 app.auto_form = False
 
 
-@app.before_request()
+@app.before_response()
 def log_request(req):
-    log.info("Before request")
-    log.info("Data: %s" % req.data)
+    log.info("Before response")
+    log.info("Data: %s", req.data)
 
 
-@app.before_request()
+@app.before_response()
 def auto_form(req):
-    """ This is own implementation of req.form paring before any POST request
+    """ This is own implementation of req.form paring before any POST response
         with own file_callback.
     """
-    if req.method_number == state.METHOD_POST:
+    if req.is_body_request or req.server_protocol == "HTTP/0.9":
         factory = StorageFactory('./upload')
         try:
             req.form = request.FieldStorage(
                 req, keep_blank_values=app.keep_blank_values,
                 strict_parsing=app.strict_parsing,
                 file_callback=factory.create)
-        except Exception:
-            log.exception()
+        except Exception as err:  # pylint: disable=broad-except
+            log.exception("Exception %s", str(err))
+            raise
 
 
 def get_crumbnav(req):
@@ -116,7 +129,7 @@ def get_header(title):
 def get_footer():
     return (
         "<hr>",
-        "<small>Copyright (c) 2013 Ondřej Tůma. See ",
+        "<small>Copyright (c) 2013-2021 Ondřej Tůma. See ",
         '<a href="http://poorhttp.zeropage.cz">poorhttp.zeropage.cz</a>'
         '</small>.',
         "</body>",
@@ -136,14 +149,19 @@ def get_variables(req):
 app.set_filter('email', r'[\w\.\-]+@[\w\.\-]+')
 
 
-def check_login(fn):
-    @wraps(fn)
+def check_login(fun):
+    """Check session cookie."""
+    @wraps(fun)
     def handler(req):
-        cookie = PoorSession(req)
-        if 'login' not in cookie.data:
+        session = PoorSession(app.secret_key)
+        try:
+            session.load(req.cookies)
+        except SessionError:
+            pass
+        if 'login' not in session.data:
             log.info('Login cookie not found.')
             redirect("/", message="Login required",)
-        return fn(req)
+        return fun(req)
     return handler
 
 
@@ -305,7 +323,7 @@ def test_varargs(req, *args):
 @app.route('/login')
 def login(req):
     log.debug("Input cookies: %s", repr(req.cookies))
-    cookie = PoorSession(req)
+    cookie = PoorSession(app.secret_key)
     cookie.data['login'] = True
     response = RedirectResponse('/')
     cookie.header(response)
@@ -315,7 +333,7 @@ def login(req):
 @app.route('/logout')
 def logout(req):
     log.debug("Input cookies: %s", repr(req.cookies))
-    cookie = PoorSession(req)
+    cookie = PoorSession(app.secret_key)
     cookie.destroy()
     response = RedirectResponse('/')
     cookie.header(response)
@@ -464,37 +482,12 @@ def not_found(req):
     return response
 
 
-@app.route('/test/headers')
-def test_headers(req):
-    return dumps(
-        {"Content-Type": (req.mime_type, req.charset),
-         "Content-Length": req.content_length,
-         "Host": req.hostname,
-         "Accept": req.accept,
-         "Accept-Charset": req.accept_charset,
-         "Accept-Encoding": req.accept_encoding,
-         "Accept-Language": req.accept_language,
-         "Accept-MimeType": {
-            "html": req.accept_html,
-            "xhtml": req.accept_xhtml,
-            "json": req.accept_json
-         },
-         "XMLHttpRequest": req.is_xhr}
-    ), "application/json"
-
-
-@app.route('/test/json', method=state.METHOD_GET_POST)
-def test_json(req):
-    return JSONResponse(status_code=418, message="I'm teapot :-)",
-                        numbers=list(range(5)),
-                        request=req.json)
-
-
-@app.route('/test/json-generator', method=state.METHOD_GET)
-def test_json_generator(req):
-    return JSONGeneratorResponse(status_code=418, message="I'm teapot :-)",
-                                 numbers=range(5),
-                                 request=req.json)
+@app.error_handler(ValueError)
+def value_error_handler(req, error):
+    """ValueError exception handler example."""
+    assert req
+    print("ValueError: ", error)
+    raise HTTPException(state.HTTP_BAD_REQUEST)
 
 
 @app.route('/test/empty')
@@ -506,26 +499,58 @@ def test_empty(req):
 
 @app.route('/yield')
 def yielded(req):
+    """Simple response generator by yield."""
     for i in range(10):
         yield b"line %d\n" % i
 
 
+@app.route('/chunked')
+def chunked(req):
+    """Generator response with Response class."""
+    def gen():
+        for i in range(10):
+            yield b"line %d\n" % i
+    return GeneratorResponse(gen(), headers={'Transfer-Encoding': 'chanked'})
+
+
+@app.route('/yield', state.METHOD_POST)
+def input_stream(req):
+    """Stream request handler"""
+    i = 0
+
+    # chunk must be read with extra method, uwsgi has own
+    chunk = uwsgi.chunked_read() if uwsgi else req.read_chunk()
+    while chunk:
+        log.info("chunk: %s", chunk)
+        if chunk != b'%d' % i:
+            raise HTTPException(state.HTTP_BAD_REQUEST)
+
+        chunk = uwsgi.chunked_read() if uwsgi else req.read_chunk()
+        i += 1
+    return EmptyResponse(state.HTTP_OK)
+
+
+@app.route('/simple')
+def simple(req):
+    """Return simple.py with FileObjResponse"""
+    assert req
+    file_ = open(__file__, 'rb')  # pylint: disable=consider-using-with
+    return FileObjResponse(file_)
+
+
 @app.route('/simple.py')
 def simple_py(req):
+    """Return simple.py with FileResponse"""
+    assert req
     return FileResponse(__file__)
 
 
-@app.after_request()
+@app.after_response()
 def log_response(req, res):
-    log.info("After request")
+    log.info("After response")
     return res
 
 
-@app.route('/timestamp')
-def get_timestamp(req):
-    return JSONResponse(timestamp=req.timestamp)
-
-
 @app.route('/internal-server-error')
 def method_raises_errror(req):
     raise RuntimeError('Test of internal server error')
@@ -538,7 +563,9 @@ def none_error_handler(req):
 
 @app.route('/bad-request')
 def bad_request(req):
-    raise HTTPException(state.HTTP_BAD_REQUEST)
+    """Endpoint raises ValueError exception."""
+    assert req
+    raise MyValueError("ValueError exception test.")
 
 
 @app.route('/forbidden')
