@@ -2,13 +2,15 @@
 
 This file could be used as known ``htdigest`` tool.
 
-.. code ::sh
+.. code:: sh
 
     python3 -m poorwsgi.digest --help
 
     # adding / updating user password
     python3 -m poorwsgi.digest -c auth.digest 'Secret Zone' user
 """
+# pylint: disable=consider-using-f-string
+
 from argparse import ArgumentParser
 from hashlib import md5, sha256
 from traceback import print_exc
@@ -223,7 +225,7 @@ class PasswordMap(defaultdict):
         if self.pathname is None:
             raise RuntimeError("No pathname was set.")
 
-        with open(self.pathname) as pwfile:
+        with open(self.pathname, encoding='utf-8') as pwfile:
             for line in pwfile:
                 username, realm, digest = line.strip().split(':')
                 self.set(realm, username, digest)
@@ -233,7 +235,7 @@ class PasswordMap(defaultdict):
         if self.pathname is None:
             raise RuntimeError("No pathname was set.")
 
-        with open(self.pathname, 'w+') as pwfile:
+        with open(self.pathname, 'w+', encoding='utf-8') as pwfile:
             for realm, vals in self.items():
                 for username, digest in vals.items():
                     pwfile.write('%s:%s:%s\n' % (username, realm, digest))
