@@ -4,14 +4,14 @@
 :Functions: parse_negotiation
 """
 # pylint: disable=too-many-lines
+# pylint: disable=consider-using-f-string
 
-from collections.abc import Mapping
-from wsgiref.headers import _formatparam  # type: ignore
 from cgi import FieldStorage as CgiFieldStorage, parse_header
 from json import loads as json_loads
 from io import BytesIO
 from time import time
-from typing import Union, Callable, Any, Iterable, List, Tuple
+from tempfile import TemporaryFile
+from typing import Union, Callable, Any, Iterable, List, Tuple, Optional
 
 import os
 import re
@@ -20,8 +20,9 @@ from logging import getLogger
 from urllib.parse import parse_qs
 from http.cookies import SimpleCookie
 
-from poorwsgi.state import methods, \
-    METHOD_POST, METHOD_PUT, METHOD_PATCH
+from poorwsgi.state import methods, HTTP_BAD_REQUEST
+from poorwsgi.headers import Headers, parse_negotiation
+from poorwsgi.response import HTTPException
 
 log = getLogger("poorwsgi")
 
@@ -32,186 +33,10 @@ RE_AUTHORIZATION = re.compile(r'(\w+)[=] ?("[^"]+"|[\w-]+)')
 # pylint: disable=unsubscriptable-object
 
 
-def parse_negotiation(value: str):
-    """Parse Content Negotiation headers to list of value, quality tuples."""
-    values = []
-    for item in value.split(','):
-        pair = item.split(';')
-        if pair[0] == item:
-            values.append((item, 1.0))
-            continue
-        try:
-            quality = float(pair[1].split('=')[1])
-        except (IndexError, ValueError):
-            quality = 1.0
-        values.append((pair[0], quality))
-    return values
-
-
-HeadersList = Union[list, tuple, set, dict]
-
-
-class Headers(Mapping):
-    """Class inherited from collections.Mapping.
-
-    As PEP 0333, resp. RFC 2616 says, all headers names must be only US-ASCII
-    character except control characters or separators. And headers values must
-    be store in string encoded in ISO-8859-1. This class methods Headers.add
-    and Headers.add_header do auto convert values from UTF-8 to ISO-8859-1
-    encoding if it is possible. So on every modification methods must be use
-    UTF-8 string.
-    """
-
-    def __init__(self, headers: HeadersList = None, strict: bool = True):
-        """Headers constructor.
-
-        Headers object could be create from list, set or tuple of pairs
-        name, value. Or from dictionary. All names or values must be
-        iso-8859-1 encodable. If not, AssertionError will be raised.
-
-        If strict is False, headers names and values are not encoded to
-        iso-8859-1. This is for input headers using only!
-        """
-        headers = headers or list()
-        if isinstance(headers, (list, tuple, set)):
-            if strict:
-                self.__headers = list(
-                    (Headers.iso88591(k), Headers.iso88591(v))
-                    for k, v in headers)
-            else:
-                self.__headers = list((k, v) for k, v in headers)
-        elif isinstance(headers, dict):
-            if strict:
-                self.__headers = list(
-                    (Headers.iso88591(k), Headers.iso88591(v))
-                    for k, v in headers.items())
-            else:
-                self.__headers = list((k, v) for k, v in headers.items())
-        else:
-            raise AssertionError("headers must be tuple, list or set "
-                                 "of str, or dict "
-                                 "(got {0})".format(type(headers)))
-    # enddef
-
-    def __len__(self):
-        """Return len of header items."""
-        return len(self.__headers)
-
-    def __getitem__(self, name: str):
-        """Return header item identified by lower name."""
-        name = Headers.iso88591(name.lower())
-        for k, val in self.__headers:
-            if k.lower() == name:
-                return val
-        raise KeyError("{0!r} is not registered".format(name))
-
-    def __delitem__(self, name: str):
-        """Delete item identied by lower name."""
-        name = Headers.iso88591(name.lower())
-        self.__headers = list(kv for kv in self.__headers
-                              if kv[0].lower() != name)
-
-    def __setitem__(self, name: str, value: str):
-        """Delete item if exist and set it's new value."""
-        del self[name]
-        self.add_header(name, value)
-
-    def __iter__(self):
-        return iter(self.__headers)
-
-    def __repr__(self):
-        return "Headers(%r)" % repr(tuple(self.__headers))
-
-    def names(self):
-        """Return tuple of headers names."""
-        return tuple(k for k, v in self.__headers)
-
-    def keys(self):
-        """Alias for names method."""
-        return self.names()
-
-    def values(self):
-        """Return tuple of headers values."""
-        return tuple(v for k, v in self.__headers)
-
-    def get_all(self, name: str):
-        """Return tuple of all values of header identifed by lower name."""
-        name = Headers.iso88591(name.lower())
-        return tuple(kv[1] for kv in self.__headers if kv[0].lower() == name)
-
-    def items(self):
-        """Return tuple of headers pairs."""
-        return tuple(self.__headers)
-
-    def setdefault(self, name: str, value: str):
-        """Set header value if not exist, and return it's value."""
-        res = self.get(name)
-        if res is None:
-            self.add_header(name, value)
-            return value
-        return res
-
-    def add(self, name: str, value: str):
-        """Set header name to value.
-
-        Duplicate names are not allowed instead of ``Set-Cookie``.
-        """
-        if name != "Set-Cookie" and name in self:
-            raise KeyError("Key %s exist." % name)
-        self.add_header(name, value)
-
-    def add_header(self, name: str, value: str, **kwargs):
-        """Extended header setting.
-
-        name is the header field to add. kwargs arguments can be used to set
-        additional parameters for the header field, with underscores converted
-        to dashes.  Normally the parameter will be added as name="value" unless
-        value is None, in which case only the name will be added.
-
-        .. code:: python
-
-            h.add_header('Content-Disposition', 'attachment',
-                         filename='image.png')
-
-        All names must be US-ASCII string except control characters
-        or separators.
-        """
-
-        parts = []
-
-        if value is not None:
-            parts.append(Headers.iso88591(value))
-
-        for k, val in kwargs.items():
-            k = Headers.iso88591(k)
-            if val is None:
-                parts.append(k.replace('_', '-'))
-            else:
-                parts.append(_formatparam(k.replace('_', '-'),
-                                          Headers.iso88591(val)))
-        self.__headers.append((Headers.iso88591(name), "; ".join(parts)))
-
-    @staticmethod
-    def iso88591(value: str) -> str:
-        """Doing automatic conversion to iso-8859-1 strings.
-
-        Converts from utf-8 to iso-8859-1 string. That means, all input value
-        of Headers class must be UTF-8 stings.
-        """
-        try:
-            if isinstance(value, str):
-                return value.encode('utf-8').decode('iso-8859-1')
-
-        except UnicodeError as err:
-            raise AssertionError("Header name/value must be iso-8859-1 "
-                                 "encoded (got {0})".format(value)) from err
-        raise AssertionError("Header name/value must be of type str "
-                             "(got {0})".format(value))
-
-
 class SimpleRequest:
     """Request proxy properties implementation - for internal use only."""
     # pylint: disable=too-many-public-methods
+    # pylint: disable=too-many-instance-attributes
     def __init__(self, environ, app):
         self.__environ = environ
         self.__app = app
@@ -238,6 +63,9 @@ class SimpleRequest:
         else:
             self.__debug = app.debug
 
+        self.__start_time = environ['REQUEST_STARTTIME']
+        self.__end_time = time()
+
     @property
     def debug(self):
         """Value of ``poor_Debug`` variable."""
@@ -326,7 +154,9 @@ class SimpleRequest:
         host = self.__environ.get('HTTP_HOST', '')
         if ':' in host:
             return int(host.split(':')[1])
-        return self.server_port
+        if self.server_scheme == 'https':
+            return 443
+        return 80
 
     @property
     def method(self):
@@ -343,12 +173,12 @@ class SimpleRequest:
     @property
     def uri(self):
         """Deprecated alias for path of the URI."""
-        return self.__environ.get('PATH_INFO')
+        return self.path
 
     @property
     def path(self):
         """Path part of url."""
-        return self.__environ.get('PATH_INFO')
+        return self.__environ.get('PATH_INFO').encode('iso-8859-1').decode()
 
     @property
     def query(self):
@@ -495,6 +325,16 @@ class SimpleRequest:
             'poor_DocumentRoot',
             self.__app.document_root)
 
+    @property
+    def start_time(self):
+        """Return timestamp when http request starts."""
+        return self.__start_time
+
+    @property
+    def end_time(self):
+        """Return timestamp when Request was created (end of __init__)."""
+        return self.__end_time
+
     def get_options(self):
         """Returns dictionary with application variables from environment.
 
@@ -514,6 +354,24 @@ class SimpleRequest:
                 options[key[4:].lower()] = val.strip()
         return options
 
+    def construct_url(self, uri: str):
+        """This function returns a fully qualified URI string.
+
+        Url is create from the path specified by uri, using the information
+        stored in the request to determine the scheme, server host name
+        and port. The port number is not included in the string if it is the
+        same as the default port 80."""
+
+        if not RE_HTTPURLPATTERN.match(uri):
+            scheme = self.forwarded_proto or self.server_scheme
+            host = self.forwarded_host or self.hostname
+            port = self.forwarded_port or self.host_port
+            if not ((port == 80 and scheme == 'http') or
+                    (port == 443 and scheme == 'https')):
+                return "%s://%s:%d%s" % (scheme, host, port, uri)
+            return "%s://%s%s" % (scheme, host, uri)
+        return uri
+
 
 class Request(SimpleRequest):
     """HTTP request object with all server elements.
@@ -531,7 +389,6 @@ class Request(SimpleRequest):
         WSGI server plus file callback for auto request body parsing.
         """
         # pylint: disable=too-many-branches, too-many-statements
-        self.__timestamp = time()
         super().__init__(environ, app)
 
         if environ.get('PATH_INFO') is None:
@@ -572,6 +429,10 @@ class Request(SimpleRequest):
             self.__file = BytesIO(self.__file.read(self.__content_length))
             self.__file.seek(0)
 
+        self.__cached_size = app.cached_size
+        self.__cached_input = None
+        self.__read_timeout = app.read_timeout
+
         # path args are set via wsgi.handler_from_table
         self.__path_args = None
 
@@ -583,12 +444,14 @@ class Request(SimpleRequest):
             self.__args = EmptyForm()
 
         # test auto json parsing
-        if app.auto_json and self.is_body_request and self.has_body \
+        if app.auto_json and \
+                (self.is_body_request or self.server_protocol == "HTTP/0.9") \
                 and self.__mime_type in app.json_mime_types:
-            self.__json = parse_json_request(self, self.__charset)
+            self.__json = parse_json_request(self.read(), self.__charset)
             self.__form = EmptyForm()
         # test auto form parsing
-        elif app.auto_form and self.is_body_request and self.has_body \
+        elif app.auto_form and \
+                (self.is_body_request or self.server_protocol == "HTTP/0.9") \
                 and self.__mime_type in app.form_mime_types:
             self.__form = FieldStorage(
                 self, keep_blank_values=app.keep_blank_values,
@@ -608,6 +471,10 @@ class Request(SimpleRequest):
         # variables for user use
         self.__user = None
         self.__api = None
+
+        # ugly hack
+        # pylint: disable=invalid-name
+        self._SimpleRequest__end_time = time()
     # enddef
 
     # -------------------------- Properties --------------------------- #
@@ -665,22 +532,22 @@ class Request(SimpleRequest):
 
     @property
     def accept_html(self):
-        """Return true if ``text/html`` mime type is in accept neogetions
+        """Return true if ``text/html`` mime type is in accept negotiations
            values.
         """
         return "text/html" in dict(self.accept)
 
     @property
     def accept_xhtml(self):
-        """Return true if ``text/xhtml`` mime type is in accept neogetions
+        """Return true if ``text/xhtml`` mime type is in accept negotiations
            values.
         """
         return "text/xhtml" in dict(self.accept)
 
     @property
     def accept_json(self):
-        """Return true if ``application/json`` mime type is in accept neogetions
-           values.
+        """Return true if ``application/json`` mime type is in accept
+           negotiations values.
         """
         return "application/json" in dict(self.accept)
 
@@ -703,16 +570,13 @@ class Request(SimpleRequest):
 
     @property
     def is_body_request(self):
-        """True if request is body request type, so it is PATCH, POST or PUT.
-        """
-        return bool(self.method_number
-                    & (METHOD_PATCH | METHOD_POST | METHOD_PUT))
+        """True if has set Content-Length more than zero."""
+        return self.__content_length > 0
 
     @property
-    def has_body(self):
-        """True if request can have body."""
-        return (self.__content_length > 0
-                or self.server_protocol == "HTTP/0.9")
+    def is_chunked_request(self):
+        """True if has set Transfer-Encoding is chunked."""
+        return self.__headers.get('Transfer-Encoding') == 'chunked'
 
     @property
     def path_args(self):
@@ -782,11 +646,6 @@ class Request(SimpleRequest):
         """
         return self.__cookies
 
-    @property
-    def timestamp(self):
-        """Return timestamp of request."""
-        return self.__timestamp
-
     @property
     def data(self):  # pylint: disable=inconsistent-return-statements
         """Returns input data from wsgi.input file.
@@ -805,7 +664,15 @@ class Request(SimpleRequest):
     @property
     def input(self):
         """Return input file, for internal use in FieldStorage"""
-        return self.__file
+        if self.__cached_input:
+            return self.__cached_input
+        if not self.__cached_size or isinstance(self.__file, BytesIO):
+            return self.__file
+        self.__cached_input = CachedInput(self.__file,
+                                          self.content_length,
+                                          self.__cached_size,
+                                          self.__read_timeout)
+        return self.__cached_input
 
     @property
     def user(self):
@@ -835,32 +702,29 @@ class Request(SimpleRequest):
         If length is not set, or if is lower then zero, Content-Length was
         be use.
         """
-        if not self.has_body:
+        if not self.is_body_request and self.server_protocol != "HTTP/0.9":
             log.error("No Content-Length found, read was failed!")
             return b''
         if -1 < length < self.__content_length:
             self.read = self.__read
             return self.read(length)
         return self.__file.read(self.__content_length)
-    # enddef
 
-    def construct_url(self, uri: str):
-        """This function returns a fully qualified URI string.
+    def read_chunk(self):
+        """Read chunk when Transfer-Encoding is `chunked`.
 
-        Url is create from the path specified by uri, using the information
-        stored in the request to determine the scheme, server host name
-        and port. The port number is not included in the string if it is the
-        same as the default port 80."""
+        Method read line with chunk size first, then read chunk and return it,
+        or raise ValueError when chunk size is in bad format.
 
-        if not RE_HTTPURLPATTERN.match(uri):
-            scheme = self.forwarded_proto or self.server_scheme
-            host = self.forwarded_host or self.hostname
-            port = self.forwarded_port or self.host_port
-            if not ((port == 80 and scheme == 'http') or
-                    (port == 443 and scheme == 'https')):
-                return "%s://%s:%d%s" % (scheme, host, port, uri)
-            return "%s://%s%s" % (scheme, host, uri)
-        return uri
+        Be sure that wsgi server allow readline from wsgi.input. For examples
+        uWSGI has extra API
+        https://uwsgi-docs.readthedocs.io/en/latest/Chunked.html
+        """
+        size = int(self.__file.readline(), base=16)
+        try:
+            return self.__file.read(size)
+        finally:
+            self.__file.readline()  # skip new line after chunk
 
     def __del__(self):
         log.debug("Request: Hasta la vista, baby.")
@@ -1031,7 +895,7 @@ class JsonList(list):
 
 
 # pylint: disable=inconsistent-return-statements
-def parse_json_request(req, charset: str = "utf-8"):
+def parse_json_request(raw: bytes, charset: str = "utf-8"):
     """Try to parse request data.
 
     Returned type could be:
@@ -1044,14 +908,99 @@ def parse_json_request(req, charset: str = "utf-8"):
     """
     # pylint: disable=inconsistent-return-statements
     try:
-        data = json_loads(req.read().decode(charset))
+        data = json_loads(raw.decode(charset))
         if isinstance(data, dict):
             return JsonDict(data.items())
         if isinstance(data, list):
             return JsonList(data)
         return data
     except BaseException as err:  # pylint: disable=broad-except
-        log.warning("Invalid request json: %s", str(err))
+        log.error("Invalid request json: %s", str(err))
+        raise HTTPException(HTTP_BAD_REQUEST, error=err) from err
+
+
+class CachedInput:
+    """
+    Wrapper around wsgi.input file, which reads data block by block.
+
+    timeout : float
+        how long to wait for new bytes in seconds
+    """
+    def __init__(self, file, size, block_size=32768,
+                 timeout: Optional[float] = 10.):
+        self.__file = file
+        self.__buffer = b''
+        self.__todo = size
+        self.__timeout = timeout
+        self.block_size = block_size
+
+    def read(self, size=-1):
+        """Compatible file read which works with internal buffer."""
+        if size < 0:
+            size = self.block_size
+
+        b_size = len(self.__buffer)
+        size = min(self.__todo, size)
+
+        if self.__buffer:
+            if b_size >= size:
+                retval = self.__buffer[:size]
+                self.__buffer = self.__buffer[size:]
+                return retval
+            size = size - b_size
+            self.__todo -= size
+            retval = self.__buffer + self.__file.read(size)
+            self.__buffer = b''
+            return retval
+
+        size = min(self.__todo, size)
+        self.__todo -= size
+        return self.__file.read(size)
+
+    def readline(self, size=-1):
+        """Compatible file read which works with internal buffer."""
+        if size < 0:
+            size = self.block_size
+
+        if not self.__buffer:
+            size = min(self.__todo, size)
+            self.__todo -= size
+            self.__buffer = self.__file.read(size)
+
+        line = b''
+        l_size = 0
+        if self.__timeout is not None:
+            times_out_at = time() + self.__timeout
+            seen_data = False
+
+        while l_size < size:
+            max_size = size-l_size
+            pos = self.__buffer.find(b'\r\n', 0, max_size)
+            if pos >= 0:
+                line += self.__buffer[:pos + 2]
+                self.__buffer = self.__buffer[pos + 2:]
+                return line
+
+            if self.__timeout is not None:
+                if self.__buffer:
+                    seen_data = True
+                elif seen_data:
+                    seen_data = False
+                    times_out_at = time() + self.__timeout
+                elif time() > times_out_at:
+                    raise TimeoutError("Timed out while receiving data")
+
+            line += self.__buffer[:max_size]
+            self.__buffer = self.__buffer[max_size:]
+            l_size = len(line)
+
+            if l_size < size:
+                n_size = min(self.__todo, max_size)
+                self.__todo -= n_size
+                self.__buffer = self.__file.read(n_size)
+
+        # no end-of-line found
+        return line
 
 
 class FieldStorage(CgiFieldStorage):
@@ -1118,6 +1067,8 @@ class FieldStorage(CgiFieldStorage):
             environ = {}
 
         self.environ = environ
+        # pylint: disable=bad-option-value,unused-private-member
+        self.__file = None
         try:
             # new interface from some 3.6 to 3.9
             super().__init__(req, headers, outerboundary, environ,
@@ -1130,16 +1081,27 @@ class FieldStorage(CgiFieldStorage):
                                  keep_blank_values, strict_parsing, limit,
                                  encoding, errors, max_num_fields)
             except TypeError:
-                # really old inteface in from some 3.6 to some 3.7
+                # really old interface from some 3.6 to some 3.7
                 super().__init__(req, headers, outerboundary, environ,
                                  keep_blank_values, strict_parsing, limit,
                                  encoding, errors)
 
     def make_file(self):
         """Return readable and writable temporary file."""
-        if 'wsgi.file_callback' in self.environ:
+        if self._binary_file and 'wsgi.file_callback' in self.environ:
             return self.environ['wsgi.file_callback'](self.filename)
-        return CgiFieldStorage.make_file(self)
+        return TemporaryFile("wb+")  # pylint: disable=consider-using-with
+
+    def read_lines(self):
+        """Internal: read lines until EOF or outerboundary."""
+        if self._binary_file and 'wsgi.file_callback' in self.environ:
+            self.file = self.make_file()
+            if self.outerboundary:
+                self.read_lines_to_outerboundary()
+            else:
+                self.read_lines_to_eof()
+        else:
+            super().read_lines()
 
     def get(self, key: str, default: Any = None):
         """Compatibility methods with dict, alias for getvalue."""
