@@ -7,7 +7,7 @@ Poor WSGI Response classes.
 :Functions:     make_response, redirect, abort
 """
 from http.client import responses
-from io import BytesIO
+from io import BytesIO, IOBase, BufferedIOBase, TextIOBase
 from os import access, R_OK, fstat
 from logging import getLogger
 from json import dumps
@@ -22,15 +22,16 @@ try:
 except ImportError:
     JSON_GENERATOR = False
 
-from poorwsgi.state import HTTP_OK, \
+from poorwsgi.state import DECLINED, HTTP_OK, \
     HTTP_MOVED_PERMANENTLY, HTTP_MOVED_TEMPORARILY, HTTP_I_AM_A_TEAPOT
-from poorwsgi.request import Headers, HeadersList
+from poorwsgi.headers import Headers, HeadersList
 
 log = getLogger('poorwsgi')
 # not in http.client.responses
 responses[HTTP_I_AM_A_TEAPOT] = "I'm a teapot"
 
 # pylint: disable=unsubscriptable-object
+# pylint: disable=consider-using-f-string
 
 
 class IBytesIO(BytesIO):
@@ -45,19 +46,12 @@ class IBytesIO(BytesIO):
         return iter(self.read_kilo, b'')
 
 
-class Response:
-    """HTTP Response object.
+class BaseResponse:
+    """Base class for response."""
 
-    This is base Response object which is process with PoorWSGI application.
-    """
-    __buffer: Union[IBytesIO, BinaryIO]
-
-    def __init__(self, data: Union[str, bytes] = b'',
-                 content_type: str = "text/html; charset=utf-8",
+    def __init__(self, content_type: str = "text/html; charset=utf-8",
                  headers: Union[Headers, HeadersList] = None,
                  status_code: int = HTTP_OK):
-        assert isinstance(data, (str, bytes)), \
-            "data is not string or bytes but %s" % type(data)
         assert isinstance(content_type, str), \
             "content_type is not string but `%s`" % content_type
         assert isinstance(status_code, int), \
@@ -79,15 +73,7 @@ class Response:
         # Status. One of state.HTTP_* values.
         self.__status_code = status_code
         self.__reason = responses[self.__status_code]
-
-        # The content length header was set automatically from buffer length.
-        if isinstance(data, bytes):
-            self.__buffer = IBytesIO(data)
-            self.__content_length = len(data)
-        else:
-            data = data.encode("utf-8")
-            self.__buffer = IBytesIO(data)
-            self.__content_length = len(data)
+        self.__done = False
 
     @property
     def status_code(self):
@@ -121,7 +107,12 @@ class Response:
 
         That is size of internal buffer.
         """
-        return self.__content_length
+        return 0
+
+    @property
+    def data(self):
+        """Return data content."""
+        return b''
 
     @property
     def headers(self):
@@ -135,23 +126,10 @@ class Response:
         else:
             self.__headers = Headers(value)
 
-    @property
-    def data(self):
-        """Return data content."""
-        self.__buffer.seek(0)
-        return self.__buffer.read()
-
     def add_header(self, name: str, value: str, **kwargs):
         """Call Headers.add_header on headers object."""
         self.__headers.add_header(name, value, **kwargs)
 
-    def write(self, data: Union[str, bytes]):
-        """Write data to internal buffer."""
-        if isinstance(data, str):
-            data = data.encode('utf-8')
-        self.__content_length += len(data)
-        self.__buffer.write(data)
-
     def __start_response__(self, start_response: Callable):
         if self.__status_code == 304:
             # Not Modified MUST NOT include other entity-headers
@@ -175,11 +153,10 @@ class Response:
                     and not self.__headers.get('Content-Type'):
                 log.info('Content-type not set!')
 
-            if self.__content_length \
+            if self.content_length \
                     and not self.__headers.get('Content-Length'):
                 self.__headers.add('Content-Length',
-                                   str(self.__content_length))
-        # endif
+                                   str(self.content_length))
 
         start_response(
             "%d %s" % (self.__status_code, self.__reason),
@@ -191,12 +168,64 @@ class Response:
         This method was called from Application object at the end of request
         for returning right value to wsgi server.
         """
-        self.__buffer.seek(0)
-        return self.__buffer
+        # pylint: disable=no-self-use
+        return b''
 
     def __call__(self, start_response: Callable):
-        self.__start_response__(start_response)
-        return self.__end_of_response__()
+        if self.__done:
+            raise RuntimeError('Response can be used only once!')
+        try:
+            self.__start_response__(start_response)
+            return self.__end_of_response__()
+        finally:
+            self.__done = True
+
+
+class Response(BaseResponse):
+    """HTTP Response object.
+
+    This is base Response object which is process with PoorWSGI application.
+
+    As Response uses BytesIO as internal cache, which is closed by WSGI
+    server, **response can be used only once!**.
+    """
+    __buffer: BufferedIOBase
+
+    def __init__(self, data: Union[str, bytes] = b'',
+                 content_type: str = "text/html; charset=utf-8",
+                 headers: Union[Headers, HeadersList] = None,
+                 status_code: int = HTTP_OK):
+        assert isinstance(data, (str, bytes)), \
+            "data is not string or bytes but %s" % type(data)
+
+        super().__init__(content_type, headers, status_code)
+
+        # The content length header was set automatically from buffer length.
+        if isinstance(data, str):
+            data = data.encode("utf-8")
+
+        self.__buffer = IBytesIO(data)
+        self.__content_length = len(data)
+
+    @property
+    def content_length(self):
+        return self.__content_length
+
+    @property
+    def data(self):
+        self.__buffer.seek(0)
+        return self.__buffer.read()
+
+    def write(self, data: Union[str, bytes]):
+        """Write data to internal buffer."""
+        if isinstance(data, str):
+            data = data.encode('utf-8')
+        self.__content_length += len(data)
+        self.__buffer.write(data)
+
+    def __end_of_response__(self):
+        self.__buffer.seek(0)
+        return self.__buffer
 
 
 class JSONResponse(Response):
@@ -208,40 +237,84 @@ class JSONResponse(Response):
                  headers: Union[Headers, HeadersList] = None,
                  status_code: int = HTTP_OK,
                  **kwargs):
-        mime_type = "application/json"
+        content_type = "application/json"
         if charset:
-            mime_type += "; charset="+charset
+            content_type += "; charset="+charset
 
-        super().__init__(dumps(kwargs), mime_type, headers, status_code)
+        super().__init__(dumps(kwargs), content_type, headers, status_code)
 
 
-class FileResponse(Response):
-    """Instead of send_file methods."""
-    def __init__(self, path: str, content_type: str = None,
+class TextResponse(Response):
+    """Simple text/plain response."""
+    def __init__(self, text: str, charset: str = "utf-8",
                  headers: Union[Headers, HeadersList] = None,
                  status_code: int = HTTP_OK):
-        if not access(path, R_OK):
-            raise IOError("Could not stat file for reading")
-        if content_type is None:     # auto mime type select
-            # pylint: disable=unused-variable
-            (content_type, encoding) = mimetypes.guess_type(path)
+        content_type = "text/plain"
+        if charset:
+            content_type += "; charset="+charset
+
+        super().__init__(text, content_type, headers, status_code)
+
+
+class FileObjResponse(BaseResponse):
+    """FileResponse returns file object direct to WSGI server.
+
+    This means, that sendfile UNIX system call can be used.
+
+    Be careful not to use a single FileReponse instance multiple times!
+    WSGI server closes file, which is returned by this response. So just
+    like Response, instance of FileResponse can be used only once!
+
+    File content is returned from current position. So Content-Length is set
+    from file system or from buffer, but minus position.
+    """
+    def __init__(self, file_obj: Union[IOBase, BinaryIO],
+                 content_type: str = None,
+                 headers: Union[Headers, HeadersList] = None,
+                 status_code: int = HTTP_OK):
+        assert file_obj.readable()
+        assert not isinstance(file_obj, TextIOBase), \
+            "file_obj must be binary stream"
         if content_type is None:     # default mime type
             content_type = "application/octet-stream"
         super().__init__(content_type=content_type,
                          headers=headers,
                          status_code=status_code)
-        self.__buffer = open(path, 'rb')
-        self.__content_length = fstat(self.__buffer.fileno()).st_size
-
-    def write(self, data):
-        raise RuntimeError("File Response can't write data")
+        self.__file = file_obj
+        if file_obj.seekable():
+            self.__pos = file_obj.tell()
+        try:
+            self.__content_length = \
+                    fstat(file_obj.fileno()).st_size - self.__pos
+        except OSError:
+            if isinstance(file_obj, BytesIO):
+                self.__content_length = \
+                        file_obj.getbuffer().nbytes - self.__pos
+            else:
+                self.__content_length = 0
+                print(type(file_obj))
+                log.debug('File object has unknown size.')
 
     # must be redefined, because self.__buffer is private attribute
     @property
     def data(self):
-        """Return data content."""
-        self.__buffer.seek(0)
-        return self.__buffer.read()
+        """Return data content.
+
+        This property works only if file_obj is seekable.
+        """
+        if self.__file.seekable():
+            self.__file.seek(self.__pos)
+            return self.__file.read()
+        log.info('File object is not seekable.')
+        return b''
+
+    @property
+    def content_length(self):
+        """Return content_length of response.
+
+        That is size of internal buffer.
+        """
+        return self.__content_length
 
     # must be redefined, because self.__buffer is private attribute
     def __end_of_response__(self):
@@ -250,12 +323,42 @@ class FileResponse(Response):
         This method was called from Application object at the end of request
         for returning right value to wsgi server.
         """
-        self.__buffer.seek(0)
-        return self.__buffer
+        if self.__file.seekable():
+            self.__file.seek(self.__pos)
+        return self.__file
 
 
-class GeneratorResponse(Response):
-    """For response, which use generator as returned value."""
+class FileResponse(FileObjResponse):
+    """FileResponse returns opened file direct to WSGI server.
+
+    This means, that sendfile UNIX system call can be used.
+
+    Be careful not to use a single FileReponse instance multiple times!
+    WSGI server closes file, which is returned by this response. So just
+    like Response, instance of FileResponse can be used only once!
+    """
+    def __init__(self, path: str, content_type: str = None,
+                 headers: Union[Headers, HeadersList] = None,
+                 status_code: int = HTTP_OK):
+        if not access(path, R_OK):
+            raise IOError("Could not stat file for reading")
+        if content_type is None:     # auto mime type select
+            # pylint: disable=unused-variable
+            (content_type, encoding) = mimetypes.guess_type(path)
+
+        # pylint: disable=consider-using-with
+        super().__init__(open(path, 'rb', buffering=0),
+                         content_type=content_type,
+                         headers=headers,
+                         status_code=status_code)
+
+
+class GeneratorResponse(BaseResponse):
+    """For response, which use generator as returned value.
+
+    Even though you can figure out iterating your generator more times, just
+    like Response, instance of GeneratorResponse can be used only once!
+    """
     def __init__(self, generator: Iterable[bytes],
                  content_type: str = "text/html; charset=utf-8",
                  headers: Union[Headers, HeadersList] = None,
@@ -265,10 +368,6 @@ class GeneratorResponse(Response):
                          status_code=status_code)
         self.__generator = generator
 
-    def write(self, data):
-        """Not possible to write data ro GeneratorResponse."""
-        raise RuntimeError("Generator Response can't write data")
-
     def __end_of_response__(self):
         return self.__generator
 
@@ -312,10 +411,10 @@ class JSONGeneratorResponse(StrGeneratorResponse):
         super().__init__(generator, mime_type, headers, status_code)
 
 
-class EmptyResponse(GeneratorResponse):
+class EmptyResponse(BaseResponse):
     """For situation, where only state is returned."""
     def __init__(self, status_code: int = HTTP_OK):
-        super().__init__((), status_code=status_code)
+        super().__init__(status_code=status_code)
 
     @property
     def headers(self):
@@ -343,22 +442,38 @@ class EmptyResponse(GeneratorResponse):
             "%d %s" % (self.status_code, self.reason), [])
 
 
+class Declined(EmptyResponse):
+    """For situation without answer.
+
+    This response is returned, when state.DECLINED was returned.
+    """
+    def __call__(self, start_response: Callable):
+        log.debug("DECLINED")
+        return ()
+
+
 class RedirectResponse(Response):
     """Redirect the browser to another location.
 
-    When permanent is true, MOVED_PERMANENTLY status code is sent to the
-    client, otherwise it is MOVED_TEMPORARILY. A short text is sent to the
-    browser informing that the document has moved (for those rare browsers that
-    do not support redirection); this text can be overridden by supplying
-    a text string.
+    A short text is sent to the browser informing that the document has moved
+    (for those rare browsers that do not support redirection); this text can
+    be overridden by supplying a text string (``message``).
+
+    When ``permanent`` or ``status_code`` is true, MOVED_PERMANENTLY
+    status code will be sent to the client, otherwise it will be
+    MOVED_TEMPORARILY. **Argument ``permanent`` and ``status_code`` as boolean
+    is deprecated. Use real status_code instead.**
     """
-    def __init__(self, location: str, permanent: bool = False,
+    # pylint: disable=too-many-arguments
+    def __init__(self, location: str,
+                 status_code: Union[int, bool] = HTTP_MOVED_TEMPORARILY,
                  message: Union[str, bytes] = b'',
-                 headers: Union[Headers, HeadersList] = None):
-        if permanent:
+                 headers: Union[Headers, HeadersList] = None,
+                 permanent: bool = False):
+        if status_code is True or permanent:
+            log.warning('Argument `permanent` is deprecated. '
+                        ' Use real status_code instead.')
             status_code = HTTP_MOVED_PERMANENTLY
-        else:
-            status_code = HTTP_MOVED_TEMPORARILY
         super().__init__(message,
                          content_type="text/plain",
                          headers=headers,
@@ -370,29 +485,6 @@ class ResponseError(RuntimeError):
     """Exception for bad response values."""
 
 
-def make_response(data: Union[str, bytes],
-                  content_type: str = "text/html; charset=utf-8",
-                  headers: Union[Headers, HeadersList] = None,
-                  status_code: int = HTTP_OK):
-    """Create response from simple values.
-
-    Data could be string, bytes, or bytes returns iterable object like file.
-    """
-    try:
-        if isinstance(data, (str, bytes)):      # "hello world"
-            return Response(data, content_type, headers, status_code)
-
-        iter(data)  # try iter data
-        return GeneratorResponse(data, content_type, headers, status_code)
-    except Exception:  # pylint: disable=broad-except
-        log.exception("Error in processing values: %s, %s, %s, %s",
-                      type(data), type(content_type), type(headers),
-                      type(status_code))
-
-    raise ResponseError(
-        "Returned data must by: <bytes|str>, <str>, <Headers|None>, <int>")
-
-
 class HTTPException(Exception):
     """HTTP Exception to fast stop work.
 
@@ -420,13 +512,61 @@ class HTTPException(Exception):
         assert isinstance(arg, (int, Response))
         super().__init__(arg, kwargs)
 
+    def make_response(self):
+        """Return or make a response if is possible."""
+        if isinstance(self.args[0], Response):
+            return self.args[0]
 
-def redirect(location: str, permanent: bool = False,
+        status_code = self.args[0]
+        if status_code == DECLINED:
+            return Declined()   # decline the connection
+        if status_code == HTTP_OK:
+            return EmptyResponse()
+        return None
+
+    @property
+    def response(self):
+        """Return response if it was set."""
+        if isinstance(self.args[0], Response):
+            return self.args[0]
+        return None
+
+
+def make_response(data: Union[str, bytes],
+                  content_type: str = "text/html; charset=utf-8",
+                  headers: Union[Headers, HeadersList] = None,
+                  status_code: int = HTTP_OK):
+    """Create response from values.
+
+    Data could be string, bytes, or bytes returns iterable object like file.
+    """
+    try:
+        if isinstance(data, (str, bytes)):      # "hello world"
+            return Response(data, content_type, headers, status_code)
+
+        iter(data)  # try iter data
+        return GeneratorResponse(data, content_type, headers, status_code)
+    except Exception:  # pylint: disable=broad-except
+        log.exception("Error in processing values: %s, %s, %s, %s",
+                      type(data), type(content_type), type(headers),
+                      type(status_code))
+
+    raise ResponseError(
+        "Returned data must by: <bytes|str>, <str>, <Headers|None>, <int>")
+
+
+def redirect(location: str,
+             status_code: Union[int, bool] = HTTP_MOVED_TEMPORARILY,
              message: Union[str, bytes] = b'',
-             headers: Union[Headers, HeadersList] = None):
-    """Raise HTTPException with RedirectResponse response."""
+             headers: Union[Headers, HeadersList] = None,
+             permanent: bool = False):
+    """Raise HTTPException with RedirectResponse response.
+
+    See RedirectResponse, with same interface for more information about
+    response.
+    """
     raise HTTPException(
-        RedirectResponse(location, permanent, message, headers))
+        RedirectResponse(location, status_code, message, headers, permanent))
 
 
 def abort(arg: Union[int, Response]):
