@@ -4,17 +4,18 @@
 :Functions: hidden, get_token, check_token
 """
 from hashlib import sha512, sha256
-from pickle import dumps, loads
+from json import dumps, loads
 from base64 import b64decode, b64encode
 from logging import getLogger
 from time import time
-from typing import Union
+from typing import Union, Dict, Any
 
 import bz2
 
 from http.cookies import SimpleCookie
 
-from poorwsgi.request import Headers
+from poorwsgi.headers import Headers
+from poorwsgi.request import Request
 from poorwsgi.response import Response
 
 log = getLogger("poorwsgi")  # pylint: disable=invalid-name
@@ -22,6 +23,7 @@ log = getLogger("poorwsgi")  # pylint: disable=invalid-name
 # pylint: disable=too-many-arguments
 # pylint: disable=too-many-instance-attributes
 # pylint: disable=unsubscriptable-object
+# pylint: disable=consider-using-f-string
 
 
 def hidden(text: Union[str, bytes], passwd: Union[str, bytes]) -> bytes:
@@ -124,7 +126,7 @@ class PoorSession:
 
     .. code:: python
 
-        sess = PoorSession(req)
+        sess = PoorSession(app.secret_key)
 
         sess.data['class'] = obj.__class__          # write to cookie
         sess.data['dict'] = obj.__dict__.copy()
@@ -147,7 +149,7 @@ class PoorSession:
                 return d
 
         obj = Obj()
-        sess = PoorSession(req)
+        sess = PoorSession(app.secret_key)
 
         sess.data['class'] = obj.__class__          # write to cookie
         sess.data['dict'] = obj.export()
@@ -156,9 +158,10 @@ class PoorSession:
         obj.import(sess.data['dict'])
     """
 
-    def __init__(self, req, expires: int = 0, max_age: int = None,
+    def __init__(self, secret_key: Union[Request, str, bytes],
+                 expires: int = 0, max_age: int = None,
                  domain: str = '', path: str = '/', secure: bool = False,
-                 same_site: bool = False, compress=bz2, SID: str = 'SESSID'):
+                 same_site: bool = False, compress=bz2, sid: str = 'SESSID'):
         """Constructor.
 
         Arguments:
@@ -181,14 +184,41 @@ class PoorSession:
                 Could be ``bz2``, ``gzip.zlib``, or any other, which have
                 standard compress and decompress methods. Or it could be
                 ``None`` to not use any compressing method.
-            SID : str
+            sid : str
                 Cookie key name.
-        """
-        if req.secret_key is None:
-            raise SessionError("poor_SecretKey is not set!")
 
-        self.__secret_key = req.secret_key
-        self.__SID = SID  # pylint: disable=invalid-name
+        .. code:: Python
+
+            session_config = {
+                'expires': 3600,  # one hour
+                'max_age': 3600,
+                'domain': 'example.net',
+                'path': '/application',
+                ̈́'secure': True,
+                'same_site': True,
+                'compress': gzip,
+                'sid': 'MYSID'
+            }
+
+            session = PostSession(app.secret_key, **config)
+            try:
+                session.load(req.cookies)
+            except SessionError as err:
+                log.error("Invalid session: %s", str(err))
+
+        *Changed in version 2.4.x*: use app.secret_key in constructor, and than
+        call load method.
+        """
+        if not isinstance(secret_key, (str, bytes)):  # backwards compatibility
+            log.warning('Do not use request in PoorSession constructor, '
+                        'see new api and call load method manually.')
+            if secret_key.secret_key is None:
+                raise SessionError("poor_SecretKey is not set!")
+            self.__secret_key = secret_key.secret_key
+        else:
+            self.__secret_key = secret_key
+
+        self.__sid = sid
         self.__expires = expires
         self.__max_age = max_age
         self.__domain = domain
@@ -198,14 +228,18 @@ class PoorSession:
         self.__cps = compress if compress is not None else NoCompress
 
         # data is session dictionary to store user data in cookie
-        self.data = {}
+        self.data: Dict[Any, Any] = {}
         self.cookie: SimpleCookie = SimpleCookie()
-        self.cookie[SID] = ''
+        self.cookie[sid] = ''
 
-        raw = None
+        if not isinstance(secret_key, (str, bytes)):  # backwards compatibility
+            self.load(secret_key.cookies)
 
-        if req.cookies and SID in req.cookies:
-            raw = req.cookies[SID].value
+    def load(self, cookies: Union[SimpleCookie, tuple]):
+        """Load session from request's cookie"""
+        if not isinstance(cookies, SimpleCookie) or self.__sid not in cookies:
+            return
+        raw = cookies[self.__sid].value
 
         if raw:
             try:
@@ -227,21 +261,21 @@ class PoorSession:
         raw = b64encode(self.__cps.compress(hidden(dumps(self.data),
                                                    self.__secret_key), 9))
         raw = raw if isinstance(raw, str) else raw.decode()
-        self.cookie[self.__SID] = raw
-        self.cookie[self.__SID]['HttpOnly'] = True
+        self.cookie[self.__sid] = raw
+        self.cookie[self.__sid]['HttpOnly'] = True
 
         if self.__domain:
-            self.cookie[self.__SID]['Domain'] = self.__domain
+            self.cookie[self.__sid]['Domain'] = self.__domain
         if self.__path:
-            self.cookie[self.__SID]['path'] = self.__path
+            self.cookie[self.__sid]['path'] = self.__path
         if self.__secure:
-            self.cookie[self.__SID]['Secure'] = True
+            self.cookie[self.__sid]['Secure'] = True
         if self.__same_site:
-            self.cookie[self.__SID]['SameSite'] = self.__same_site
+            self.cookie[self.__sid]['SameSite'] = self.__same_site
         if self.__expires:
-            self.cookie[self.__SID]['expires'] = self.__expires
+            self.cookie[self.__sid]['expires'] = self.__expires
         if self.__max_age is not None:
-            self.cookie[self.__SID]['Max-Age'] = self.__max_age
+            self.cookie[self.__sid]['Max-Age'] = self.__max_age
 
         return raw
 
@@ -251,12 +285,12 @@ class PoorSession:
         Be sure, that data can't be changed:
         https://stackoverflow.com/a/5285982/8379994
         """
-        self.cookie[self.__SID]['expires'] = -1
+        self.cookie[self.__sid]['expires'] = -1
         if self.__max_age is not None:
-            self.cookie[self.__SID]['Max-Age'] = -1
-        self.cookie[self.__SID]['HttpOnly'] = True
+            self.cookie[self.__sid]['Max-Age'] = -1
+        self.cookie[self.__sid]['HttpOnly'] = True
         if self.__secure:
-            self.cookie[self.__SID]['Secure'] = True
+            self.cookie[self.__sid]['Secure'] = True
 
     def header(self, headers: Union[Headers, Response] = None):
         """Generate cookie headers and append it to headers if it set.
