@@ -6,23 +6,26 @@
 """
 # pylint: disable=too-many-lines
 # pylint: disable=unsubscriptable-object
+# pylint: disable=consider-using-f-string
 
 from os import path, access, R_OK, environ
 from collections import OrderedDict
 from logging import getLogger
 from hashlib import md5, sha256
-from typing import List, Union, Callable, Optional
+from typing import List, Union, Callable, Optional, Type
+from time import time
 
 import re
 
-from poorwsgi.state import HTTP_OK, DECLINED, \
+from poorwsgi.state import \
     METHOD_GET, METHOD_POST, METHOD_HEAD, methods, \
-    HTTP_METHOD_NOT_ALLOWED, HTTP_NOT_FOUND, HTTP_FORBIDDEN
+    HTTP_METHOD_NOT_ALLOWED, HTTP_NOT_FOUND, HTTP_FORBIDDEN, \
+    deprecated
 from poorwsgi.request import Request, SimpleRequest
 from poorwsgi.results import default_states, not_implemented, \
     internal_server_error, directory_index, debug_info
-from poorwsgi.response import Response, HTTPException, EmptyResponse, \
-    FileResponse, make_response, ResponseError
+from poorwsgi.response import BaseResponse, HTTPException, \
+    FileObjResponse, FileResponse, make_response, ResponseError
 
 log = getLogger("poorwsgi")
 
@@ -42,7 +45,7 @@ AUTH_DIGEST_ALGORITHMS = {
 
 def to_response(response):
     """handler response to application response."""
-    if isinstance(response, Response):
+    if isinstance(response, BaseResponse):
         return response
 
     if not isinstance(response, tuple):
@@ -98,13 +101,18 @@ class Application():
         # http state handlers: {HTTP_NOT_FOUND: {METHOD_GET: my_404_handler}}
         self.__shandlers = {}
 
+        # exception handlers: {ValueError: {METHOD_GET: my_value_handler}}
+        self.__ehandlers = OrderedDict()
+
         # -- Application variable
         self.__config = {
             'auto_args': True,
             'auto_form': True,
             'auto_json': True,
             'auto_data': True,
-            'data_size': 32768,
+            'cached_size': 65365,
+            'data_size': 65365,
+            'read_timeout': 10,
             'keep_blank_values': 0,
             'strict_parsing': 0,
             'file_callback': None,
@@ -153,7 +161,7 @@ class Application():
 
         return "(?P<%s>%s)" % (groups[0], regex)
 
-    def __convertor(self, _filter):
+    def __converter(self, _filter):
         _filter = str(_filter).lower()
         _filter = ':re:' if _filter[:4] == ':re:' else _filter
         try:
@@ -195,17 +203,17 @@ class Application():
 
     @property
     def before(self):
-        """Tuple of table with before-request handlers.
+        """Tuple of table with before-response handlers.
 
-        See Application.before_request.
+        See Application.before_response.
         """
         return tuple(self.__before)
 
     @property
     def after(self):
-        """Tuple of table with after-request handlers.
+        """Tuple of table with after-response handlers.
 
-        See Application.after_request.
+        See Application.after_response.
         """
         return tuple(self.__after)
 
@@ -235,12 +243,20 @@ class Application():
 
     @property
     def states(self):
-        """Copy of table with http state aka error handlers.
+        """Copy of table with http state handlers.
 
         See Application.http_state
         """
         return self.__shandlers.copy()
 
+    @property
+    def errors(self):
+        """Copy of table with exception handlers.
+
+        See Application.error_handler
+        """
+        return self.__ehandlers.copy()
+
     @property
     def auto_args(self):
         """Automatic parsing request arguments from uri.
@@ -294,6 +310,18 @@ class Application():
     def auto_data(self, value: Union[int, bool]):
         self.__config['auto_data'] = bool(value)
 
+    @property
+    def cached_size(self):
+        """Enabling cached_size for faster POST request.
+
+        Default value is 65365.
+        """
+        return self.__config['cached_size']
+
+    @cached_size.setter
+    def cached_size(self, value: int):
+        self.__config['cached_size'] = value
+
     @property
     def data_size(self):
         """Size limit for Request.data property.
@@ -312,7 +340,7 @@ class Application():
         """Automatic parsing cookies from request headers.
 
         If it is True (default) and Cookie request header was set,
-        SimpleCookie object was paresed to Request property cookies.
+        SimpleCookie object was parsed to Request property cookies.
         """
         return self.__config['auto_cookies']
 
@@ -399,8 +427,8 @@ class Application():
     def file_callback(self):
         """File callback use as parameter when parsing request body.
 
-        Default is None. Values could be a class or factory which got's
-        filename from request body and have file compatibile interface.
+        Default is None. Values could be a class or factory which got
+        filename from request body and have file compatible interface.
         """
         return self.__config['file_callback']
 
@@ -408,11 +436,21 @@ class Application():
     def file_callback(self, value: Callable):
         self.__config['file_callback'] = value
 
+    @property
+    def read_timeout(self):
+        """Gets a timeout (in seconds) used for file receiving"""
+        return self.__config["read_timeout"]
+
+    @read_timeout.setter
+    def read_timeout(self, timeout: float):
+        """Sets a timeout (in seconds) used for file receiving"""
+        self.__config["read_timeout"] = timeout
+
     @property
     def json_mime_types(self):
         """Copy of json mime type list.
 
-        Containt list of strings as json mime types, which is use for
+        Contains list of strings as json mime types, which is use for
         testing, when automatics Json object is create from request body.
         """
         return self.__config['json_mime_types']
@@ -506,20 +544,20 @@ class Application():
     def form_mime_types(self):
         """Copy of form mime type list.
 
-        Containt list of strings as form mime types, which is use for
+        Contains list of strings as form mime types, which is use for
         testing, when automatics Form object is create from request body.
         """
         return self.__config['form_mime_types']
 
-    def set_filter(self, name: str, regex: str, convertor: Callable = str):
-        r"""Create new filter or overwrite builtins.
+    def set_filter(self, name: str, regex: str, converter: Callable = str):
+        r"""Create new filter or overwrite built-ins.
 
         name : str
             Name of filter which is used in route or set_route method.
         regex : str
             Regular expression which used for filter.
-        convertor : function
-            Convertor function or class, which gets string in input. Default is
+        converter : function
+            Converter function or class, which gets string in input. Default is
             str function, which call __str__ method on input object.
 
         .. code:: python
@@ -527,85 +565,122 @@ class Application():
             app.set_filter('uint', r'\d+', int)
         """
         name = ':'+name if name[0] != ':' else name
-        self.__filters[name] = (regex, convertor)
+        self.__filters[name] = (regex, converter)
 
+    @deprecated("use before_response instead")
     def before_request(self):
-        """Append hendler to call before each request.
+        """Deprecated, use before_request instead."""
 
-        This is decorator for function to call before each request.
-
-        .. code:: python
-
-            @app.before_request()
-            def before_each_request(req):
-                print("Request coming")
-        """
         def wrapper(fun):
-            self.add_before_request(fun)
+            self.add_before_response(fun)
             return fun
         return wrapper
 
-    def add_before_request(self, fun: Callable):
-        """Append handler to call before each request.
+    def before_response(self):
+        """Append handler to call before each response.
 
-        Method adds function to list functions which is call before each
-        request.
+        This is decorator for function to call before each response.
 
         .. code:: python
 
-            def before_each_request(req):
-                print("Request coming")
+            @app.before_response()
+            def before_each_response(req):
+                print("Response coming")
+        """
+        def wrapper(fun):
+            self.add_before_response(fun)
+            return fun
+        return wrapper
 
-            app.add_before(before_each_request)
+    @deprecated("use add_before_response instead")
+    def add_before_request(self, fun: Callable):
+        """Deprecated, use add_before_response instead."""
+        self.add_before_response(fun)
+
+    def add_before_response(self, fun: Callable):
+        """Append handler to call before each response.
+
+        Method adds function to list functions which is call before each
+        response.
+
+        .. code:: python
+
+            def before_each_response(req):
+                print("Response coming")
+
+            app.add_before_response(before_each_response)
         """
         if self.__before.count(fun):
             raise ValueError("%s is in list yet" % str(fun))
         self.__before.append(fun)
 
+    @deprecated("use pop_before_response instead")
     def pop_before_request(self, fun: Callable):
-        """Remove handler added by add_before_request or before_request."""
+        """Deprecated, use pop_before_response instead."""
+        self.pop_before_response(fun)
+
+    def pop_before_response(self, fun: Callable):
+        """Remove handler added by add_before_response or before_response."""
         if not self.__before.count(fun):
             raise ValueError("%s is not in list" % str(fun))
         self.__before.remove(fun)
 
+    @deprecated("use after_response instead")
     def after_request(self):
-        """Append handler to call after each request.
+        """Deprecated, use after_response instead."""
+        def wrapper(fun):
+            self.add_after_response(fun)
+            return fun
+        return wrapper
 
-        This decorator append function to be called after each request,
+    def after_response(self):
+        """Append handler to call after each response.
+
+        This decorator append function to be called after each response,
         if you want to use it redefined all outputs.
 
         .. code:: python
 
-            @app.after_each_request()
-            def after_each_request(request, response):
-                print("Request out")
+            @app.after_response()
+            def after_each_response(request, response):
+                print("Response out")
                 return response
         """
         def wrapper(fun):
-            self.add_after_request(fun)
+            self.add_after_response(fun)
             return fun
         return wrapper
 
+    @deprecated("use add_after_response instead")
     def add_after_request(self, fun: Callable):
-        """Append handler to call after each request.
+        """Deprecated, use add_after_response instead."""
+        self.add_after_response(fun)
+
+    def add_after_response(self, fun: Callable):
+        """Append handler to call after each response.
 
         Method for direct append function to list functions which are called
-        after each request.
+        after each response.
 
         .. code:: python
 
-            def after_each_request(request, response):
-                print("Request out")
+            def after_each_response(request, response):
+                print("Response out")
                 return response
 
-            app.add_after_request(after_each_request)
+            app.add_after_response(after_each_response)
         """
         if self.__after.count(fun):
             raise ValueError("%s is in list yet" % str(fun))
         self.__after.append(fun)
 
+    @deprecated("use pop_after_response instead")
     def pop_after_request(self, fun: Callable):
-        """Remove handler added by add_before_request or before_request."""
+        """Deprecated, use pop_after_response instead."""
+        self.pop_after_response(fun)
+
+    def pop_after_response(self, fun: Callable):
+        """Remove handler added by add_after_response or after_response."""
         if not self.__before.count(fun):
             raise ValueError("%s is not in list" % str(fun))
         self.__after.remove(fun)
@@ -635,7 +710,7 @@ class Application():
                     method: int = METHOD_HEAD | METHOD_GET):
         """Set default handler.
 
-        Set fun default handler for http method called befor error_not_found.
+        Set fun default handler for http method called before error_not_found.
 
         .. code:: python
 
@@ -714,10 +789,10 @@ class Application():
         """
         if re_filter.search(uri):
             r_uri = re_filter.sub(self.__regex, uri) + '$'
-            convertors = tuple((g[0], self.__convertor(g[1]))
+            converters = tuple((g[0], self.__converter(g[1]))
                                for g in (m.groups()
                                          for m in re_filter.finditer(uri)))
-            self.set_regular_route(r_uri, fun, method, convertors, uri)
+            self.set_regular_route(r_uri, fun, method, converters, uri)
         else:
             if uri not in self.__handlers:
                 self.__handlers[uri] = {}
@@ -779,8 +854,8 @@ class Application():
 
     def set_regular_route(self, uri: str, fun: Callable,
                           method: int = METHOD_HEAD | METHOD_GET,
-                          convertors=(), rule: str = None):
-        r"""Set hanlder for uri defined by regular expression.
+                          converters=(), rule: str = None):
+        r"""Set handler for uri defined by regular expression.
 
         Another way to add fn as handler for uri defined by regular expression.
         See Application.regular_route documentation for details.
@@ -799,12 +874,12 @@ class Application():
             self.__rhandlers[r_uri] = {}
         for val in methods.values():
             if method & val:
-                self.__rhandlers[r_uri][val] = (fun, convertors, rule)
+                self.__rhandlers[r_uri][val] = (fun, converters, rule)
 
     def pop_regular_route(self, uri: str, method: int):
-        """Pop handler and convertors for uri and method from handlers table.
+        """Pop handler and converters for uri and method from handlers table.
 
-        For mor details see Application.pop_route.
+        For more details see Application.pop_route.
         """
         r_uri = re.compile(uri, re.U)
         handlers = self.__rhandlers.get(r_uri, {})
@@ -820,13 +895,13 @@ class Application():
 
     def http_state(self, status_code: int,
                    method: int = METHOD_HEAD | METHOD_GET | METHOD_POST):
-        """Wrap function to handle http status codes like http errors.
+        """Wrap function to handle http status codes.
 
         .. code:: python
 
             @app.http_state(state.HTTP_NOT_FOUND)
             def page_not_found(req):
-                return "Your request %s not found." % req.uri, "text/plain"
+                return "Your page %s was not found." % req.path, "text/plain"
         """
         def wrapper(fun):
             self.set_http_state(status_code, fun, method)
@@ -835,24 +910,59 @@ class Application():
 
     def set_http_state(self, status_code: int, fun: Callable,
                        method: int = METHOD_HEAD | METHOD_GET | METHOD_POST):
-        """Set fn as handler for http state code and method."""
+        """Set function as handler for http state code and method."""
         if status_code not in self.__shandlers:
             self.__shandlers[status_code] = {}
         for val in methods.values():
             if method & val:
                 self.__shandlers[status_code][val] = fun
 
-    def pop_http_state(self, status_code, method: int):
-        """Pop handerl for http state and method.
+    def pop_http_state(self, status_code: int, method: int):
+        """Pop handler for http state and method.
 
-        As Application.pop_route, for pop multimethod handler, you must call
+        As Application.pop_route, for pop multi-method handler, you must call
         pop_http_state for each method.
         """
         handlers = self.__shandlers.get(status_code, {})
         return handlers.pop(method)
 
-    def error_from_table(self, req, status_code, **kwargs):
-        """Internal method, which is called if error was accured.
+    def error_handler(
+            self, error: Type[Exception],
+            method: int = METHOD_HEAD | METHOD_GET | METHOD_POST):
+        """Wrap function to handle exceptions.
+
+        .. code:: python
+
+            @app.error_handler(ValueError)
+            def value_error(req):
+                return "Values %s are not correct." % req.args, "text/plain"
+        """
+        def wrapper(fun):
+            self.set_error_handler(error, fun, method)
+            return fun
+        return wrapper
+
+    def set_error_handler(
+            self, error: Type[Exception], fun: Callable,
+            method: int = METHOD_HEAD | METHOD_GET | METHOD_POST):
+        """Set function as handler for exception and method."""
+        if error not in self.__ehandlers:
+            self.__ehandlers[error] = {}
+        for val in methods.values():
+            if method & val:
+                self.__ehandlers[error][val] = fun
+
+    def pop_error_handler(self, error: Type[Exception], method: int):
+        """Pop handler for http state and method.
+
+        As Application.pop_route, for pop multi-method handler, you must call
+        pop_http_state for each method.
+        """
+        handlers = self.__ehandlers.get(error, {})
+        return handlers.pop(method)
+
+    def state_from_table(self, req: SimpleRequest, status_code: int, **kwargs):
+        """Internal method, which is called if another http state has occurred.
 
         If status code is in Application.shandlers (fill with http_state
         function), call this handler.
@@ -862,8 +972,12 @@ class Application():
             try:
                 handler = self.__shandlers[status_code][req.method_number]
                 req.error_handler = handler
-                self.handler_from_before(req)  # call before handlers now
                 return handler(req, **kwargs)
+            except HTTPException as http_err:
+                response = http_err.make_response()
+                if response:
+                    return response
+                return internal_server_error(req)
             except Exception:  # pylint: disable=broad-except
                 return internal_server_error(req)
         elif status_code in default_states:
@@ -873,7 +987,35 @@ class Application():
         else:
             return not_implemented(req, status_code)
 
-    def handler_from_default(self, req):
+    def error_from_table(self, req: SimpleRequest, error: Exception, **kwargs):
+        """Internal method, which is called when exception was raised."""
+
+        handler = None
+        for error_type, hdls in self.__ehandlers.items():
+            if isinstance(error, error_type) \
+                    and req.method_number in hdls:
+                handler = hdls[req.method_number]
+                break
+
+        if handler:
+            try:
+                req.error_handler = handler
+                return handler(req, error, **kwargs)
+
+            except HTTPException as http_err:
+                response = http_err.make_response()
+                if response:
+                    return response
+                status_code = http_err.args[0]
+                kwargs = http_err.args[1]
+                return to_response(
+                        self.state_from_table(req, status_code, **kwargs))
+
+            except Exception:  # pylint: disable=broad-except
+                return internal_server_error(req)
+        return None
+
+    def handler_from_default(self, req: SimpleRequest):
         """Internal method, which is called if no handler is found."""
         req.uri_rule = '/*'
         if req.method_number in self.__dhandlers:
@@ -882,10 +1024,10 @@ class Application():
             return self.__dhandlers[req.method_number](req)
 
         self.handler_from_before(req)       # call before handlers now
-        log.error("404 Not Found: %s %s", req.method, req.uri)
+        log.error("404 Not Found: %s %s", req.method, req.path)
         raise HTTPException(HTTP_NOT_FOUND)
 
-    def handler_from_before(self, req):
+    def handler_from_before(self, req: SimpleRequest):
         """Internal method, which run all before (pre_proccess) handlers.
 
         This method was call before end-point route handler.
@@ -893,7 +1035,7 @@ class Application():
         for fun in self.__before:
             fun(req)
 
-    def handler_from_table(self, req):
+    def handler_from_table(self, req: Request):
         """Call right handler from handlers table (fill with route function).
 
         If no handler is fined, try to find directory or file if Document Root,
@@ -902,10 +1044,11 @@ class Application():
         """
         # pylint: disable=too-many-return-statements
         # static routes
-        if req.uri in self.__handlers:
-            if req.method_number in self.__handlers[req.uri]:
-                handler = self.__handlers[req.uri][req.method_number]
-                req.uri_rule = req.uri      # nice variable for before handlers
+
+        if req.path in self.__handlers:
+            if req.method_number in self.__handlers[req.path]:
+                handler = self.__handlers[req.path][req.method_number]
+                req.uri_rule = req.path  # nice variable for before handlers
                 req.uri_handler = handler
                 self.handler_from_before(req)  # call before handlers now
                 return handler(req)       # call right handler now
@@ -914,18 +1057,18 @@ class Application():
             raise HTTPException(HTTP_METHOD_NOT_ALLOWED)
 
         # regular expression
-        for ruri in self.__rhandlers.keys():
-            match = ruri.match(req.uri)
+        for ruri in self.__rhandlers:
+            match = ruri.match(req.path)
             if match and req.method_number in self.__rhandlers[ruri]:
-                handler, convertors, rule = \
+                handler, converters, rule = \
                     self.__rhandlers[ruri][req.method_number]
                 req.uri_rule = rule or ruri.pattern
                 req.uri_handler = handler
-                if convertors:
-                    # create OrderedDict from match insead of dict for
-                    # convertors applying
+                if converters:
+                    # create OrderedDict from match inside of dict for
+                    # converters applying
                     req.path_args = OrderedDict(
-                        (g, c(v))for ((g, c), v) in zip(convertors,
+                        (g, c(v))for ((g, c), v) in zip(converters,
                                                         match.groups()))
                     self.handler_from_before(req)   # call before handlers now
                     return handler(req, *req.path_args.values())
@@ -938,10 +1081,10 @@ class Application():
         if req.document_root and \
                 req.method_number & (METHOD_HEAD | METHOD_GET):
             rfile = "%s%s" % (req.document_root,
-                              path.normpath("%s" % req.uri))
+                              path.normpath("%s" % req.path))
 
             if not path.exists(rfile):
-                if req.debug and req.uri == '/debug-info':      # work if debug
+                if req.debug and req.path == '/debug-info':  # work if debug
                     req.uri_rule = '/debug-info'
                     req.uri_handler = debug_info
                     self.handler_from_before(req)  # call before handlers now
@@ -952,13 +1095,13 @@ class Application():
             if path.isfile(rfile) and access(rfile, R_OK):
                 req.uri_rule = '/*'
                 self.handler_from_before(req)      # call before handlers now
-                log.info("Return file: %s", req.uri)
+                log.info("Return file: %s", req.path)
                 return FileResponse(rfile)
 
             # return directory index
             if req.document_index and path.isdir(rfile) \
                     and access(rfile, R_OK):
-                log.info("Return directory: %s", req.uri)
+                log.info("Return directory: %s", req.path)
                 req.uri_rule = '/*'
                 req.uri_handler = directory_index
                 self.handler_from_before(req)      # call before handlers now
@@ -967,7 +1110,7 @@ class Application():
             raise HTTPException(HTTP_FORBIDDEN)
         # req.document_root
 
-        if req.debug and req.uri == '/debug-info':
+        if req.debug and req.path == '/debug-info':
             req.uri_rule = '/debug-info'
             req.uri_handler = debug_info
             self.handler_from_before(req)          # call before handlers now
@@ -980,10 +1123,11 @@ class Application():
 
         This method create Request object, call handlers from
         Application.before, uri handler (handler_from_table), default handler
-        (Application.defaults) or error handler (Application.error_from_table),
+        (Application.defaults) or error handler (Application.state_from_table),
         and handlers from Application.after.
         """
         # pylint: disable=method-hidden,too-many-branches
+        env['REQUEST_STARTTIME'] = time()
         request = None
 
         try:
@@ -991,18 +1135,15 @@ class Application():
             args = self.handler_from_table(request)
             response = to_response(args)
         except HTTPException as http_err:
-            if isinstance(http_err.args[0], Response):
-                response = http_err.args[0]
-            else:
+            if request is None:
+                request = SimpleRequest(env, self)
+
+            response = http_err.make_response()
+            if not response:
                 status_code = http_err.args[0]
                 kwargs = http_err.args[1]
-                if status_code == DECLINED:
-                    return ()   # decline the connection
-                if status_code == HTTP_OK:
-                    response = EmptyResponse()
-                else:
-                    response = to_response(
-                        self.error_from_table(request, status_code, **kwargs))
+                response = to_response(
+                        self.state_from_table(request, status_code, **kwargs))
         except (ConnectionError, SystemExit) as err:
             log.warning(str(err))
             log.warning('   ***   You should ignore next error   ***')
@@ -1010,7 +1151,7 @@ class Application():
         except ResponseError:
             log.error("Bad returned value from %s", request.uri_handler)
             try:
-                response = to_response(self.error_from_table(request, 500))
+                response = to_response(self.state_from_table(request, 500))
             except Exception:  # pylint: disable=broad-except
                 log.error("Bad returned value from %s", request.error_handler)
                 response = internal_server_error(request)
@@ -1021,7 +1162,9 @@ class Application():
                 request = SimpleRequest(env, self)
 
             try:
-                response = to_response(self.error_from_table(request, 500))
+                response = self.error_from_table(request, err)
+                if not response:
+                    response = to_response(self.state_from_table(request, 500))
             except Exception:  # pylint: disable=broad-except
                 log.error("Bad returned value from %s", request.error_handler)
                 response = internal_server_error(request)
@@ -1031,12 +1174,14 @@ class Application():
             for fun in self.__after:
                 __fn = fun
                 response = to_response(fun(request, response))
-        except BaseException:  # pylint: disable=broad-except
+        except BaseException as err:  # pylint: disable=broad-except
             log.error("Handler %s from %s returns invalid data or crashed",
                       __fn, __fn.__module__)
-            response = to_response(self.error_from_table(request, 500))
+            response = self.error_from_table(request, err)
+            if not response:
+                response = to_response(self.state_from_table(request, 500))
 
-        if isinstance(response, FileResponse) and \
+        if isinstance(response, FileObjResponse) and \
                 "wsgi.file_wrapper" in env:     # need working fileno method
             return env['wsgi.file_wrapper'](response(start_response))
         return response(start_response)         # return bytes generator
